//
// Autonomous Functions

// ----------------------------------------------------------------------

void autoEngine() {
  // main scheduler for autonomous mode
  switch (autoTaskPnt) {
    case 0:
      // engine setup starts here
      headSweep = -1; // switch OFF head functions
      speedTQ = 0; // enforce 'Tortoise' mode at start
      RGBEnh = 1; setRGBEnh(); setAllRGBLeds(0,0,0);
      autoHeadPnt = 0; autoHeadAng = 0; autoHeadInc = 1;
      autoHeadDel = 5; autoHeadCnt = autoHeadDel;
      autoTaskPnt = 1; delay(500); break;
    case 1:
      // all task assignments from this state
      autoMoveHead();
      break;
  }
}

// ----------------------------------------------------------------------

void autoMoveHead() {
  // move head functions called from autoEngine
  // autobody functions were removed as unnecessary
  /* 
    *  Head scans right/left/right/left whilst stationary
    *  If clear move forward for a random period
    *  Scan at slower rate whilst moving
    *  If not clear rotate and scan until clear 
    *  If clear move forward for a random period 
    *  If not clear after full rotate then stop
  */
  int zObj = 0; // objects detected cound
  int zObjL = 0; // objects detected cound
  int zObjR = 0; // objects detected cound
  int zP = 0; // any loop counter
  switch (autoHeadPnt) {
    case 0:
      // move head to 0 point before scanning
//      Serial.println("Mode 0 = static scan");
      if (RGB != 6) {RGB = 6; RGBmode = 0;}
      JoyX = JoyCentre; JoyY = JoyCentre; // stop all movement
      if (servoAng[0] > 0) {
        // not at 0 so move there
        SetServoAngle(0, servoAng[0] - 1); echoTO = 2;
        autoHeadAng = servoAng[0]; autoHeadAngLast = autoHeadAng;
      } else {
        // now at 0 so move onto scanning
        autoHeadDel = 1; autoHeadPhase = 0;
        autoHeadCnt = 1; autoHeadInc = 1;
        autoHeadAngLast = -1; // enable angle based range triggering
        autoHeadPnt = 1;
      } break;
    case 1:
      // scan R/L/R/L whilst ranging at 6 angles
      // hexabot is not moving
      autoHeadCnt--;
      if (autoHeadCnt < 1) {
        // increment sweep position after a delay
        autoHeadCnt = autoHeadDel;
        autoHeadAng = autoHeadAng + autoHeadInc;
        if (autoHeadAng >= 90) {
          autoHeadPhase++; // sweep up phase complete
          autoHeadAng = 90; autoHeadInc = -autoHeadInc;
        } else if (autoHeadAng <= 0) {
          autoHeadPhase++; // sweep down phase complete
          autoHeadAng = 0; autoHeadInc = -autoHeadInc;
        }
        SetServoAngle(0, autoHeadAng);
        if (autoHeadPhase > 3) {autoHeadPnt = 2;} // test for 4 sweeps
      } break;
    case 2:
      // was anything detected in the sweep?
      for (int zP=0; zP < 6; zP++) {
        if (rangeArray[zP] <= objTrig) {zObj++;} // count objects detected
      }
      if (zObj < 1) {
        // nothing detected so start moving forward
        autoHeadPnt = 10;
      } else {
        // one or more objects in the array
        autoHeadPnt = 30;
      } break;

      
    case 10:
      // nothing detected in static scan so move forward for a random
      // distance whilst scanning
//      Serial.println("Mode 10 - walk forward");
      if (RGB != 2) {RGB = 2; RGBmode = 0;}
      JoyX = JoyCentre;
//      JoyY = JoyCentreUL + 40 + random(58);
      JoyY = 245;
      autoHeadDel = 3; autoHeadCnt = 0;
      autoBodyPCnt = 1600 + random (4000); // walk for 8s - 20s
      autoHeadPnt = 11; rangeLedON = 0; break;
    case 11:
      // moving forward whilst scanning for a period
      // remove steering offsets if they have been applied
      autoSteerCnt--;
      if (autoSteerCnt < 1) {
        autoSteerCnt = 9; // decreasing steer away bias last for ~4s
        if (JoyX > JoyCentre) {JoyX--;}
        else if (JoyX < JoyCentre) {JoyX++;}
        Serial.print("JX="); Serial.println(JoyX);
      }
      // sweep head
      autoHeadCnt--;
      if (autoHeadCnt < 1) {
        // increment sweep position after a delay
        autoHeadCnt = autoHeadDel;
        autoHeadAng = autoHeadAng + autoHeadInc;
        if (autoHeadAng >= 90) {
          autoHeadAng = 90; autoHeadInc = -autoHeadInc;
        } else if (autoHeadAng <= 0) {
          autoHeadAng = 0; autoHeadInc = -autoHeadInc;
        } SetServoAngle(0, autoHeadAng);
      }
      if (echoTO == 2) {
        // was anything detected whilst range sweeping?
        // test long range for steering
        zObjL = 0;
        for (zP=0; zP < 3; zP++) {
          if (rangeArray[zP] <= objSteer) {zObjL++;} // count objects detected
        } zObjR = 0;
        for (zP=3; zP < 6; zP++) {
          if (rangeArray[zP] <= objSteer) {zObjR++;} // count objects detected
        }
        if (zObjL > zObjR) {
          // turn towards right to avoid obstacle on left
          JoyX = 245;
        } else if (zObjL < zObjR) {
          // turn towards left to avoid obstacle on right
          JoyX = 10;
        }
        // test close range for STOP and change
        zObj = 0;
        for (zP=0; zP < 6; zP++) {
          if (rangeArray[zP] <= objTrig) {zObj++;} // count objects detected
        }
        if (zObj > 0) {autoHeadPnt = 0;} // object detected so stop
      }
      // test for end of walking period?
      autoBodyPCnt--; if (autoBodyPCnt < 1) {autoHeadPnt = 20;}
      break;

      
    case 20:
      // reached end of walking period without detecting an object
      // so perform a random turn L/R
//      Serial.println("Mode 20 - random turn");
      JoyY = JoyCentre; moveMode = 0; // stop any forward motion
      if (random(2) < 1) {
        // turn right
//        JoyX = JoyCentreUL + 20 + random(40);
        JoyX = 245;
        if (RGB != 3) {RGB = 3; RGBmode = 0;}
      } else {
        // turn left
//        JoyX = JoyCentreLL - 20 - random(40);
        JoyX = 10;
        if (RGB != 5) {RGB = 2; RGBmode = 0;}
      } // at this speed it takes ~5s to turn 90 degrees
      // at 5ms loop rate, 1s = 200 cycles
      autoBodyPCnt = 400 + random(1600); // period 2s - 10s
      autoHeadDel = 3; autoHeadCnt = 0;
      autoHeadPnt = 21; break;
    case 21:
      // turning whilst scanning for a period
      autoHeadCnt--;
      if (autoHeadCnt < 1) {
        // increment sweep position after a delay
        autoHeadCnt = autoHeadDel;
        autoHeadAng = autoHeadAng + autoHeadInc;
        if (autoHeadAng >= 90) {
          autoHeadAng = 90; autoHeadInc = -autoHeadInc;
        } else if (autoHeadAng <= 0) {
          autoHeadAng = 0; autoHeadInc = -autoHeadInc;
        } SetServoAngle(0, autoHeadAng);
      }
      // test for end of turning period?
      autoBodyPCnt--; if (autoBodyPCnt < 1) {autoHeadPnt = 0;}
      break;


    case 30:
      // one or more obstacles in field of view so rotate until clear
      JoyY = JoyCentre; moveMode = 0; // stop any forward motion
      JoyX = JoyCentreLL + 60; // set default rotation right
//      Serial.println("Mode 30 - obstacles!");
      if (rangeArray[0] > rangeArray[5]) {
        // object on right, so turn left
        JoyX = JoyCentreLL - 60;
      } else if (rangeArray[0] < rangeArray[5]) {
        // object on the left, so turn right
        JoyX = JoyCentreUL + 60;
      } else if (rangeArray[1] > rangeArray[4]) {
        // object on right, so turn left
        JoyX = JoyCentreLL - 60;
      } else if (rangeArray[1] < rangeArray[4]) {
        // object on the left, so turn right
        JoyX = JoyCentreUL + 60;
      } else if (rangeArray[2] > rangeArray[3]) {
        // object on right, so turn left
        JoyX = JoyCentreLL - 60;
      } else if (rangeArray[2] < rangeArray[3]) {
        // object on the left, so turn right
        JoyX = JoyCentreUL + 60;
      }
      autoHeadDel = 1; autoHeadCnt = autoHeadDel;
      autoHeadInc = 1; if (autoHeadAng > 0) {autoHeadInc = -1;}
      autoHeadPnt = 31; break;
    case 31:
      // keep turning and scanning looking for a clear gap
      autoHeadCnt--;
      if (autoHeadCnt < 1) {
        // increment sweep position after a delay
        autoHeadCnt = autoHeadDel;
        autoHeadAng = autoHeadAng + autoHeadInc;
        if (autoHeadAng >= 90) {
          autoHeadPhase++; // sweep up phase complete
          autoHeadAng = 90; autoHeadInc = -autoHeadInc;
        } else if (autoHeadAng <= 0) {
          autoHeadPhase++; // sweep down phase complete
          autoHeadAng = 0; autoHeadInc = -autoHeadInc;
        }
        SetServoAngle(0, autoHeadAng);
      }
      // was anything detected in the sweep?
      zObj = 0;
      for (int zP=0; zP < 6; zP++) {
        if (rangeArray[zP] <= objTrig) {zObj++;} // count objects detected
      }
//      Serial.print(zObj); Serial.println(" Objects");
      if (zObj < 1) {autoHeadPnt = 10;} // nothing detected so walk
      break;
  }

  
  //
  // check to see if a range has been triggered
  // if triggered keep a rolling average
  // this code must be BEFORE the triggering code
  echoTO++;
  if (echoTO == 1) {
    // HCSR04 triggered in previous cycle
    // check for range echo
    if (intFlag > 0) {
      // echo received
      rangeVal = (intMicros/58)-9;
//      Serial.print("Range="); Serial.println(rangeVal);
    } else {
      // no echo so assume out of range
      intFlag = 1; // mask further interrupts
      rangeVal = 50; // max 50cm range to have no effect
    }
    // rolling average ranging data over 3 sweeps
    rangeVal = (rangeVal + (rangeArray[rangeFlag] * 2))/3;
    rangeArray[rangeFlag] = rangeVal;
    if (rangeVal <= objSteer) {
      rangeLEDs[rangeFlag] = 0x100000; rangeLedON = 12;
    } else {
      rangeLEDs[rangeFlag] = 2; // default weak blue
      if (rangeLedON > 0) {rangeLedON--;}
    }
  }
  //
  // trigger a range measurement whenever at a specific angle
  if (autoHeadAng != autoHeadAngLast) {
    autoHeadAngLast = autoHeadAng;
    switch (autoHeadAng) {
      // test angles 0-18-36-54-72-90 in up/down scan
      case 0: trigHCSR04 (); echoTO = 0; rangeFlag = 0; break;
      case 18: trigHCSR04 (); echoTO = 0; rangeFlag = 1; break;
      case 36: trigHCSR04 (); echoTO = 0; rangeFlag = 2; break;
      case 54: trigHCSR04 (); echoTO = 0; rangeFlag = 3; break;
      case 72: trigHCSR04 (); echoTO = 0; rangeFlag = 4; break;
      case 90: trigHCSR04 (); echoTO = 0; rangeFlag = 5; break;
    }
  }
}

