
//
// Various Functions

// ----------------------------------------------------------------------

void bluetoothTick() {
  // received Bluetooth '+' tick character
  bluetoothCnt = 200; // set time-out for 1s with 5ms loop cycle
  BTRx = 1; // enable time-out
}

// ----------------------------------------------------------------------

void clrColArray(){
  // sets all array colour values to 0
  for (int zP = 0; zP < 16; zP++) {colRed[zP] = 0; colGrn[zP] = 0; colBlu[zP] = 0;}
}

// ----------------------------------------------------------------------

void CLS(int zN) {
  // clear the serial monitor by writing zN blank lines
  for (int zP = 0; zP < zN; zP++) {Serial.println("");}
}

// ----------------------------------------------------------------------

void clsCol(){
  // sets all array colour values to 0 and clears the display
  clrColArray(); showColArray(0);
}

// ----------------------------------------------------------------------

void decodeKey(int zkeyVal) {
  // decodes zkeyVal and excutes on '.'
  keyChar = char(zkeyVal);
  int zNF = 0;
  switch (keyChar) {
    case '.': doCmd(); zNF = 1; break;
    case 'z': STOP(); zNF = 1; break;
    case 'Z': STOP(); zNF = 1; break;
    case '!': Reset(); zNF = 3; break;
    case '+': bluetoothTick(); zNF = 3; break;
    case '0': extendCmdVal(0); zNF = 1; break;
    case '1': extendCmdVal(1); zNF = 1; break;
    case '2': extendCmdVal(2); zNF = 1; break;
    case '3': extendCmdVal(3); zNF = 1; break;
    case '4': extendCmdVal(4); zNF = 1; break;
    case '5': extendCmdVal(5); zNF = 1; break;
    case '6': extendCmdVal(6); zNF = 1; break;
    case '7': extendCmdVal(7); zNF = 1; break;
    case '8': extendCmdVal(8); zNF = 1; break;
    case '9': extendCmdVal(9); zNF = 1; break;
  }
  if (zNF == 0) {
    if (cmdMode == ' ') {
      // test for new Command Mode char?
      switch (keyChar) {
        case 'b': cmdMode = 'B'; break;
        case 'B': cmdMode = 'B'; break;
        case 'p': cmdMode = 'P'; break;
        case 'P': cmdMode = 'P'; break;
        case 's': cmdMode = 'S'; break;
        case 'S': cmdMode = 'S'; break;
        case 'x': cmdMode = 'X'; break;
        case 'X': cmdMode = 'X'; break;
        case 'y': cmdMode = 'Y'; break;
        case 'Y': cmdMode = 'Y'; break;
      } cmdType = ' '; cmdVal = 0;
    } else {
      // test for Command Type char?
      switch (keyChar) {
        case 'a': cmdType = 'A'; break;
        case 'A': cmdType = 'A'; break;
        case 'd': cmdType = 'D'; break;
        case 'D': cmdType = 'D'; break;
        case 'f': cmdType = 'F'; break;
        case 'F': cmdType = 'F'; break;
        case 'k': cmdType = 'K'; break;
        case 'K': cmdType = 'K'; break;
        case 'l': cmdType = 'L'; break;
        case 'L': cmdType = 'L'; break;
        case 'n': cmdType = 'N'; break;
        case 'N': cmdType = 'N'; break;
        case 'o': cmdType = 'O'; break;
        case 'O': cmdType = 'O'; break;
        case 's': cmdType = 'S'; break;
        case 'S': cmdType = 'S'; break;
        case 't': cmdType = 'T'; break;
        case 'T': cmdType = 'T'; break;
        case 'v': cmdType = 'V'; break;
        case 'V': cmdType = 'V'; break;
      }
    }
  }
  if (zNF == 3) {
    // clear received data
    cmdMode = ' '; cmdType = ' '; cmdVal = 0;
  }
}

// ----------------------------------------------------------------------

void doCmd() {
  // a '.' has been received so execute command if valid
  Serial.print(cmdMode); Serial.print(cmdType); Serial.print(cmdVal); Serial.println('.');
  if (doWait < 1) {
    switch (cmdMode) {
      case ' ': break;
      case 'B':
        switch (cmdType) {
          case 'N': {doCmdButton(cmdVal); break;}
        } break;
      case 'P':
        switch (cmdType) {
          case 'K': {SetToParkPos(cmdVal,1); break;}
          case 'L': {SetToLegPos(cmdVal); break;}
        } break;
      case 'S':
        switch (cmdType) {
          case 'D': {SetServoSwpDelta(); break;}
          case 'F': {SetServoFreq(); break;}
          case 'O': {SetServoOE(cmdVal); break;}
          case 'S': {SetServoSweep(); break;}
          case 'T': {SetServoTarget(); break;}
          case 'V': {SetServoValue(); break;}
        } break;
      case 'X':
        if ((cmdVal >= 0) && (cmdVal <= 255)) {
          JoyX = cmdVal; // only adopt in-range values
        } break;
      case 'Y':
        if ((cmdVal >= 0) && (cmdVal <= 255)) {
          JoyY = cmdVal; // only adopt in-range values
        } break;
    }
    // now reset the variables
    cmdMode = ' '; cmdType = ' '; cmdVal = 0;
  } else {doWait--;}
}

// ----------------------------------------------------------------------

void doCmdButton(int zB) {
  // received a button command from controller
  // button references are row+column. ie. 00 = top left button
  switch (zB) {
    case 00:
      // toggle seroOE PWM enable
      if (servoOE < 1) {moveMode = 10;} // adopt a standing position
      break;
    case 01:
      // toggle 'Tortoise' mode
      if (speedTQ > 0) {speedTQ = 0;} else {speedTQ = 1;}
      break;
    case 02:
      // toggle seroOE PWM enable
      if (servoOE > 0) {servoOE = 0;} else {servoOE = 1;}
      SetServoOE(servoOE); break;
    case 10:
      // change raised mode
      switch (legMode) {
        case 0: legMode = 2; break;
        case 1: legMode = 3; break;
        case 2: legMode = 0; break;
        case 3: legMode = 1; break;
      } break;
    case 11:
      // change stride mode
      switch (legMode) {
        case 0: legMode = 1; break;
        case 1: legMode = 0; break;
        case 2: legMode = 3; break;
        case 3: legMode = 2; break;
      } break;
    case 12:
      // toggle seroOE PWM enable
      if (servoOE > 0) {servoOE = 0;} else {servoOE = 1;}
      SetServoOE(servoOE); break;
    case 20:
      // toggle head sweep ON/OFF
      if (headSweep < 0) {headSweep = 99; rangeTIR = 180;} else {headSweep = -1; SetServoAngle(0,45);}
      break;
    case 21:
      // toggle RGB LED patterns ON/OFF
      if (RGBEnh > 0) {RGBEnh = 0;} else {RGBEnh = 1; setRGBEnh();}
      setRGBEnh(); break;
    case 22:
      // toggle seroOE PWM enable
      if (servoOE > 0) {servoOE = 0;} else {servoOE = 1;}
      SetServoOE(servoOE); break;
    case 30:
      // enter track mode
      if (autoMode != 3) {doSw2Action(3);} else {doSw2Action(99);}
      break;
    case 31:
      // full autonomouse button, toggle mode
      if (autoMode != 4) {doSw2Action(4);} else {doSw2Action(99);}
      break;
    case 32:
      Reset();; break;
  }
}

// ----------------------------------------------------------------------

void doSw0Action(int zSwCnt0) {
  // respond with switch 0 actions
  switch (zSwCnt0) {
    case 0:
      // NOP task
      loopEn = 0; break;
    case 1:
      // move head to centre position with no ultrasonics
      if (upDwnFlg > 0) {
        flashAllLEDs(zSwCnt0,32,0,0);
        SetServoAngle(0, 45); headSweep = -1; RGBon(1); loopEn = 0;
      } else{loopEn = 100;} break;
    case 2:
      // move head side to side if stood up
      if (upDwnFlg > 0) {
        flashAllLEDs(zSwCnt0,32,0,0);
        headSweep = 99; rangeTIR = 180;
        moveMode = 9; loopEn = 0; RGBon(0);
      } else{loopEn = 100;} break;
    case 3:
      // centre head in ranging mode
      if (upDwnFlg > 0) {
        flashAllLEDs(zSwCnt0,32,0,0);
        headSweep = 1999;
        moveMode = 9; loopEn = 0; RGBon(0);
      } else{loopEn = 100;} break;
    case 99:
      // long switch 0 press
      if (upDwnFlg > 0) {
        // sit HexaBot down and RESET
        setAllRGBLeds (0,32,0);
        delay(500);
        setAllRGBLeds (0,0,0);
        Reset(); // sit and reset
      } else {
        // stand HexaBot up then REST
        setAllRGBLeds (32,0,0);
        waitSwDwn();
        setAllRGBLeds (0,0,0); RGBon(1);
        delay(500);
        SetServoOE(0); // switch ON PWM
        moveMode = 10; loopEn = 0; // move to 'stand'
      } break;
  }
}

// ----------------------------------------------------------------------

void doSw1Action(int zSwCnt1) {
  // respond with switch 1 actions
  if (upDwnFlg > 0) {
    // only respond to switches if stood up
    switch (zSwCnt1) {
      case 0:
        // NOP task
        loopEn = 0; break;
      case 1:
        // normal 'slow' mode
        flashAllLEDs(zSwCnt1,0,32,0);
        legMode = 0; loopEn = 0;
        Serial.println("Leg Mode=0");
        break;
      case 2:
        // normal 'quick' mode
        flashAllLEDs(zSwCnt1,0,32,0);
        legMode = 1; loopEn = 0;
        Serial.println("Leg Mode=1");
        break;
      case 3:
        // carpet 'slow' mode
        flashAllLEDs(zSwCnt1,0,32,0);
        legMode = 2; loopEn = 0;
        Serial.println("Leg Mode=2");
        break;
      case 4:
        // carpet 'quick' mode
        flashAllLEDs(zSwCnt1,0,32,0);
        legMode = 3; loopEn = 0;
        Serial.println("Leg Mode=3");
        break;
    }
  } else{loopEn = 100;}
}

// ----------------------------------------------------------------------

void doSw2Action(int zSwCnt2) {
  // respond with switch 2 actions
  if (upDwnFlg > 0) {
    // only respond to switches if stood up
    switch (zSwCnt2) {
      case 0:
        // NOP task
        loopEn = 0; break;
      case 1:
        // enter back-off mode 'quick'
        RGB = 0; flashAllLEDs(zSwCnt2,0,0,32);
        headSweep = 1999; moveMode = 9; // centre head
        autoMode = 1; speedTQ = 1; loopEn = 0;
        Serial.println("Automode=1");
        break;
      case 2:
        // enter chase mode 'quick'
        RGB = 0; flashAllLEDs(zSwCnt2,0,0,32);
        headSweep = 1999; moveMode = 9; // centre head
        autoMode = 2; speedTQ = 1; loopEn = 0;
        Serial.println("Automode=2");
        break;
      case 3:
        // enter chase mode 'quick'
        RGB = 0; flashAllLEDs(zSwCnt2,0,0,32);
        headSweep = 1999; moveMode = 9; // centre head
        autoMode = 3; speedTQ = 1; loopEn = 0;
        Serial.println("Automode=3");
        break;
      case 4:
        // enter autonomous mode
        RGB = 1; flashAllLEDs(zSwCnt2,0,0,32);
        autoMode = 4; autoTaskPnt = 0;
        speedTQ = 0; loopEn = 0;
        Serial.println("Automode=4");
        break;
      case 99:
        // long press, so clear automodes
        RGB = 1; setAllRGBLeds(0,0,32); delay(300); setAllRGBLeds(0,0,0);
        Serial.println("Automode=0");
        JoyX = 128; JoyY = 128; // remove any auto-demand
        waitSwDwn(); autoMode = 0; loopEn = 0;
        SetServoAngle(0,45); headSweep = -1; // centre and disable head
        moveMode = modeToRest; // reset position
        break;
    }
  } else{loopEn = 100;}
}

// ----------------------------------------------------------------------

void echoInt() {
  // interrupt service routine
  // flag used to indicate occurance and prevent multiple readings
  if (intFlag < 1) {intMicros = micros() - intMicros; intFlag = 1;}
}

// ----------------------------------------------------------------------

void extendCmdVal(int zVal) {
  // adds a new digit to the right-hand end of cmdVal
  cmdVal = (cmdVal * 10) + zVal;  
}

// ----------------------------------------------------------------------

void flashAllLEDs(int zCnt,int zR,int zG,int zB) {
  // flash all LEDs, zCnt times, with colour ZR,ZG,ZB
  for (int zL=0;zL<zCnt;zL++) {
    setAllRGBLeds(zR,zG,zB); delay(75);
    setAllRGBLeds(0,0,0); delay(250);
  }
}

// ----------------------------------------------------------------------

void getRange() {
  // ultrasonic range finder is ON so run these tasks
  switch (rangeFlag) {
      // there is no '0' state in these tasks, 0 = OFF
      case 1:
        trigHCSR04 (); break;
      case 3:
        if (intFlag > 0) {
          // an interrupt occured during the 10ms wait period
//          Serial.print("Time=");
//          Serial.print(intMicros);
//          Serial.println("us");
//          Serial.print("Distance=");
//          Serial.print((intMicros/58)-9);
//          Serial.println("cm");
        } else {
          intFlag = 1; // prevent interrupts from recording
//          Serial.println("No Range");
        } break;
      case 40:
        rangeFlag = 0; break; // repeat every 200ms
  }
  rangeFlag++;
}

// ----------------------------------------------------------------------

void loadColArray(long zC0,long zC1,long zC2,long zC3,long zC4,long zC5,long zC6,long zC7,long zC8,long zC9,long zC10,long zC11,long zC12,long zC13,long zC14,long zC15) {
  // loads 16 hexadecimal colours into the colour array
  colBlu[0] = zC0 & 0xFF; colGrn[0] = (zC0>>8) & 0xFF; colRed[0] = (zC0>>16) & 0xFF;
  colBlu[1] = zC1 & 0xFF; colGrn[1] = (zC1>>8) & 0xFF; colRed[1] = (zC1>>16) & 0xFF;
  colBlu[2] = zC2 & 0xFF; colGrn[2] = (zC2>>8) & 0xFF; colRed[2] = (zC2>>16) & 0xFF;
  colBlu[3] = zC3 & 0xFF; colGrn[3] = (zC3>>8) & 0xFF; colRed[3] = (zC3>>16) & 0xFF;
  colBlu[4] = zC4 & 0xFF; colGrn[4] = (zC4>>8) & 0xFF; colRed[4] = (zC4>>16) & 0xFF;
  colBlu[5] = zC5 & 0xFF; colGrn[5] = (zC5>>8) & 0xFF; colRed[5] = (zC5>>16) & 0xFF;
  colBlu[6] = zC6 & 0xFF; colGrn[6] = (zC6>>8) & 0xFF; colRed[6] = (zC6>>16) & 0xFF;
  colBlu[7] = zC7 & 0xFF; colGrn[7] = (zC7>>8) & 0xFF; colRed[7] = (zC7>>16) & 0xFF;
  colBlu[8] = zC8 & 0xFF; colGrn[8] = (zC8>>8) & 0xFF; colRed[8] = (zC8>>16) & 0xFF;
  colBlu[9] = zC9 & 0xFF; colGrn[9] = (zC9>>8) & 0xFF; colRed[9] = (zC9>>16) & 0xFF;
  colBlu[10] = zC10 & 0xFF; colGrn[10] = (zC10>>8) & 0xFF; colRed[10] = (zC10>>16) & 0xFF;
  colBlu[11] = zC11 & 0xFF; colGrn[11] = (zC11>>8) & 0xFF; colRed[11] = (zC11>>16) & 0xFF;
  colBlu[12] = zC12 & 0xFF; colGrn[12] = (zC12>>8) & 0xFF; colRed[12] = (zC12>>16) & 0xFF;
  colBlu[13] = zC13 & 0xFF; colGrn[13] = (zC13>>8) & 0xFF; colRed[13] = (zC13>>16) & 0xFF;
  colBlu[14] = zC14 & 0xFF; colGrn[14] = (zC14>>8) & 0xFF; colRed[14] = (zC14>>16) & 0xFF;
  colBlu[15] = zC15 & 0xFF; colGrn[15] = (zC15>>8) & 0xFF; colRed[15] = (zC15>>16) & 0xFF;
}

// ----------------------------------------------------------------------

void   LoadDefArrays() {
  // load the calibration values into the arrays
  // if you fit a new servo you need to relax values to determine limits
  // relax value limits are LL=120, UL=590
  servoVal[0]  = 350; servoVLL[0]  = 258; servoVUL[0]  = 442; // Set
  servoVal[1]  = 363; servoVLL[1]  = 165; servoVUL[1]  = 560; // Set
  servoVal[2]  = 307; servoVLL[2]  = 187; servoVUL[2]  = 427; // Set
  servoVal[3]  = 356; servoVLL[3]  = 168; servoVUL[3]  = 544; // Set
  servoVal[4]  = 370; servoVLL[4]  = 253; servoVUL[4]  = 486; // Set
  servoVal[5]  = 338; servoVLL[5]  = 130; servoVUL[5]  = 545; // Set
  servoVal[6]  = 296; servoVLL[6]  = 176; servoVUL[6]  = 416; // Set
  servoVal[7]  = 358; servoVLL[7]  = 155; servoVUL[7]  = 560; // Set
  servoVal[8]  = 366; servoVLL[8]  = 245; servoVUL[8]  = 486; // Set
  servoVal[9]  = 338; servoVLL[9]  = 135; servoVUL[9]  = 540; // Set
  servoVal[10] = 305; servoVLL[10] = 189; servoVUL[10] = 420; // Set
  servoVal[11] = 330; servoVLL[11] = 135; servoVUL[11] = 525; // Set
  servoVal[12] = 370; servoVLL[12] = 251; servoVUL[12] = 488; // Set
  servoVal[13] = 348; servoVLL[13] = 120; servoVUL[13] = 576; // Not used
  servoVal[14] = 348; servoVLL[14] = 120; servoVUL[14] = 576; // Not used
  servoVal[15] = 348; servoVLL[15] = 120; servoVUL[15] = 576; // Not used
}

// ----------------------------------------------------------------------

void readBTChar() {
  // read a character from the Bluetooth device serial port
  btVal = BTserial.read();
  if (btVal != -1) {
    Serial.print(char(btVal));
    decodeKey(btVal);
  }
}

// ----------------------------------------------------------------------

void readKey() {
  // reads a key from the keyboard annd reacts accordingly
  if (doWait < 1) {
    keyVal = Serial.read();
    if (keyVal != -1) {decodeKey(keyVal);}
  }
}

// ----------------------------------------------------------------------

void Reset() {
  // Reset the HexBot code
  SetServoOE(0); SetServoStartPos(30); // move to sit position
  SetDefaults();
  Serial.println("RESET!");
  RunPOST();
}

// ----------------------------------------------------------------------

void RGBon(int zT) {
  // enable the RGB LED task zT, if zT = 0 stops RGB mode
  RGBmode = 0; RGB = zT;
}

// ----------------------------------------------------------------------

void RGBtasks() {
  // main task scheduler for the RGB LEDs
  switch(RGB) {
    case 1: RGBtask1(); break; // rotate a blue wave
    case 2: RGBtask2(); break; // moving forwards
    case 3: RGBtask3(); break; // rotating to the right
    case 4: RGBtask4(); break; // moving backwards
    case 5: RGBtask5(); break; // rotating to the left
    case 6: RGBtask6(); break; // blue head tracker band
  }
}

// ----------------------------------------------------------------------

void RGBtask1() {
  // rotate a blue wave
  switch (RGBmode) {
    case 0:
      loadColArray(7,8,9,10,11,12,13,18,0,0,1,2,3,4,5,6);
      RGBmode++; break;
    case 1:
      showColArray(0); RotColArrRht(0,15); RGBCntr = 15;
      RGBmode++; break;
    case 2:
      RGBCntr--;
      if (RGBCntr < 1) {RGBmode = 1;} // end of time-out, so loop
      break;
  }
}

// ----------------------------------------------------------------------

void RGBtask2() {
  // move is going forward, so two flowing waves front to back
  // R/L rates affected by steerFlag slow the respective side
  int zJoyY;
  switch (RGBmode) {
    case 0:
      loadColArray(0x080800,0x080000,0x060000,0x040000,0x020000,0x010000,0x000000,0x181800,   0x181800,0x000000,0x010000,0x020000,0x040000,0x060000,0x080000,0x080800);
      RGBmode = 1; RGBSteerFlag = 0; break;
    case 1:
      if (rangeLedON > 0) {showColMasked();} else {showColArray(0);}
      if (steerFlag == 0) {
        // no steering so normal LED sweep front to back
        RotColArrLft(0,7); RotColArrRht(8,15);
      } else if(steerFlag > 0) {
        // steering to the right
        RotColArrLft(0,7); // normal speed
        if (RGBSteerFlag == 1) {
          // slow down RH side
          RotColArrRht(8,15);}
      } else {
        // steering to the left
        RotColArrRht(8,15); // normal speed
        if (RGBSteerFlag == 1) {
          // slow down LH side
          RotColArrLft(0,7);}
      }
      RGBSteerFlag--; if (RGBSteerFlag < 1) {RGBSteerFlag = 3;}
      zJoyY = max(JoyCentreUL,JoyY);
      RGBCntr = 20-((zJoyY - JoyCentreUL)/6); RGBmode = 2;
      RGBStrDir = steerFlag; break;
    case 2:
      RGBCntr--;
      if (RGBCntr < 1) {
        // end of time-out, so loop
        RGBmode = 1; // default mode unless out of synch
        if ((RGBStrDir != steerFlag) && (steerFlag == 0)) {
          // steering has centralised so resynch RGB LEDs
          RGBmode = 0;
        }
      } break;
  }
}

// ----------------------------------------------------------------------

void RGBtask3() {
  // move is rotating right, so circular wave
  int zJoyX;
  switch (RGBmode) {
    case 0:
      loadColArray(0x000000,0x010000,0x020000,0x040000,0x060000,0x080000,0x080800,0x101000,    0x000000,0x010000,0x020000,0x040000,0x060000,0x080000,0x080800,0x101000);
      RGBmode++; break;
    case 1:
      showColArray(0); RotColArrRht(0,15);
      zJoyX= max(JoyCentreUL,JoyX);
      RGBCntr = 20-((zJoyX - JoyCentreUL)/6); RGBmode++; break;
    case 2:
      RGBCntr--;
      if (RGBCntr < 1) {RGBmode = 1;} // end of time-out, so loop
      break;
  }
}

// ----------------------------------------------------------------------

void RGBtask4() {
  // move is going backward, so two flowing waves back to front
  int zJoyY;
  switch (RGBmode) {
    case 0:
      loadColArray(0x181800,0x000000,0x010000,0x020000,0x040000,0x060000,0x080000,0x080800,    0x080800,0x080000,0x060000,0x040000,0x020000,0x010000,0x000000,0x181800);
      RGBmode = 1; RGBSteerFlag = 0; break;
    case 1:
      showColArray(0);
      if (steerFlag == 0) {
        RotColArrLft(8,15); RotColArrRht(0,7);
      } else if(steerFlag > 0) {
        // steering to the right
        RotColArrRht(0,7); // normal speed
        if (RGBSteerFlag == 1) {
          // slow down RH side
          RotColArrLft(8,15);}
      } else {
        // steering to the left
        RotColArrLft(8,15); // normal speed
        if (RGBSteerFlag == 1) {
          // slow down LH side
          RotColArrRht(0,7);}
      }
      RGBSteerFlag--; if (RGBSteerFlag < 1) {RGBSteerFlag = 3;}
      zJoyY= min(JoyCentreLL,JoyY);
      RGBCntr = 20-((JoyCentreLL - zJoyY)/6); RGBmode=2; break;
    case 2:
      RGBCntr--;
      if (RGBCntr < 1) {
        // end of time-out, so loop
        RGBmode = 1; // default mode unless out of synch
        if ((RGBStrDir != steerFlag) && (steerFlag == 0)) {
          // steering has centralised so resynch RGB LEDs
          RGBmode = 0;
        }
      } break;
  }
}

// ----------------------------------------------------------------------

void RGBtask5() {
  // move is rotating left, so circular wave
  int zJoyX;
  switch (RGBmode) {
    case 0:
      loadColArray(0x181800,0x080800,0x080000,0x060000,0x040000,0x020000,0x010000,0x000000,    0x181800,0x080800,0x060000,0x040000,0x020000,0x020000,0x010000,0x000000);
      RGBmode++; break;
    case 1:
      showColArray(0); RotColArrLft(0,15);
      zJoyX= min(JoyCentreLL,zJoyX);
      // speed from slow = 20 to fast = 4
      RGBCntr = 20-((JoyCentreLL-JoyX)/6); RGBmode++; break;
    case 2:
      RGBCntr--;
      if (RGBCntr < 1) {RGBmode = 1;} // end of time-out, so loop
      break;
  }
}

// ----------------------------------------------------------------------

void RGBtask6() {
  // rotate a band tracking the head
  switch (RGBmode) {
    case 0:
      // set up blue band
      loadColArray(0,0,0,0,0,0,4,16,    16,4,0,0,0,0,0,0);
      RGBmode++; break;
    case 1:
      // position band with head angle      
      showColArray((45 - servoAng[0])/10);
      RGBCntr = 5; RGBmode++; break;
    case 2:
      RGBCntr--;
      if (RGBCntr < 1) {RGBmode = 1;} // end of time-out, so loop
      break;
  }
}

// ----------------------------------------------------------------------

void RotColArrLft(int zS, int zF) {
  // rotate the contents of the colour array counter-clockwise
  int zPP;
  uint8_t zR = colRed[zS]; uint8_t zG = colGrn[zS]; uint8_t zB = colBlu[zS];
  for (int zP = zS; zP < zF+1; zP++) {
    zPP = zP+1; colRed[zP] = colRed[zPP]; colGrn[zP] = colGrn[zPP]; colBlu[zP] = colBlu[zPP];
  } colRed[zF] = zR; colGrn[zF] = zG; colBlu[zF] = zB;
}

// ----------------------------------------------------------------------

void RotColArrRht(int zS, int zF) {
  // rotate the contents of the colour array clockwise
  int zPP;
  uint8_t zR = colRed[zF]; uint8_t zG = colGrn[zF]; uint8_t zB = colBlu[zF];
  for (int zP = zF; zP > zS; zP--) {
    zPP = zP-1; colRed[zP] = colRed[zPP]; colGrn[zP] = colGrn[zPP]; colBlu[zP] = colBlu[zPP];
  } colRed[zS] = zR; colGrn[zS] = zG; colBlu[zS] = zB;
}

// ----------------------------------------------------------------------

void RunPOST() {
  // Runs once at power-up or Reset
  CLS(20);
  Serial.println("Starting POST...");
  // descrete Red Green Blue LED sequence
  setLEDs (HIGH, HIGH, HIGH); delay(200);
  setLEDs (LOW, LOW, LOW); delay(100);
  setLEDs (HIGH, LOW, LOW); delay(200);
  setLEDs (LOW, HIGH, LOW); delay(200);
  setLEDs (LOW, LOW, HIGH); delay(200);
  setLEDs (LOW, LOW, LOW); delay(100);
  setLEDs (HIGH, HIGH, HIGH); delay(200);
  setLEDs (LOW, LOW, LOW);
  if (onceFlag > 0) {
    SetServoOE(0); SetServoStartPos(30);
  }
  Serial.println("Servo Default Positions Set.");
  Serial.print("Waiting.");
  delay(200); // allow time for servos to reach default positions
  Serial.print("."); delay(200);
  Serial.print("."); delay(200);
  Serial.print("."); delay(200);
  Serial.println("."); delay(200);
  SetServoOE(1); // disable PWM
  int zBright = 16;
  // set RGB LEDs Red
  for (int zLED = 0; zLED < 16; zLED++) {
    strip.setPixelColor(zLED,strip.Color(zBright, 0, 0));
  } strip.show(); delay(200);
  // set RGB LEDs Green
  for (int zLED = 0; zLED < 16; zLED++) {
    strip.setPixelColor(zLED,strip.Color(0,zBright, 0));
  } strip.show(); delay(200);
  // set RGB LEDs Blue
  for (int zLED = 0; zLED < 16; zLED++) {
    strip.setPixelColor(zLED,strip.Color(0, 0, zBright));
  } strip.show(); delay(200);
  for (int zLED = 0; zLED < 16; zLED++) {
    strip.setPixelColor(zLED,strip.Color(0, 0, 4));
  } strip.show();
  Serial.println("POST Complete!");
  next5ms = micros() + 5000; // set 5ms timer
  nextMove = micros() + moveInterval; // set 'Move' variable timer
  RGBon(1); // start RGB LEDs rotating
  onceFlag = 0; // POST has been run at start-up
}

// ----------------------------------------------------------------------

void scanSwitches() {
  // scan switches on Blue Tooth board every 5ms
  ledState0 = LOW; ledState1 = LOW; ledState2 = LOW;
  // read switch 0
  swRead0 = digitalRead(Pin_SW0);
  if (swRead0 != swState0) {
    // switch '0' has changed
    if (swRead0 == LOW) {swCnt0++; swTO0 = 0;}
  } swState0 = swRead0; swTO0++;
  if (swRead0 == LOW) {ledState0 = HIGH; loopEn = 2000; setAllRGBLeds(4,0,0);}
  if (swCnt0 > 0) {
    if (swState0 == LOW) {
      // test for switch held down for > 2s
      if (swTO0 > 400) {
        Serial.println("SW0 long press!");
        doSw0Action(99);
        swCnt0 = 0;
      }
    } else {
      // time-out switch count action after 1s
      if (swTO0 > 200) {
        Serial.print("SW0 Cnt=");
        Serial.println(swCnt0);
        doSw0Action(swCnt0);
        swCnt0 = 0;
      }
    }
  }
  // read switch 1
  swRead1 = digitalRead(Pin_SW1);
  if (swRead1 != swState1) {
    // switch '1' has changed
    if (swRead1 == LOW) {swCnt1++; swTO1= 0;}
  } swState1 = swRead1; swTO1++;
  if (swRead1 == LOW) {ledState1 = HIGH; loopEn = 2000; setAllRGBLeds(0,4,0);}
  if (swCnt1 > 0) {
    if (swState1 == LOW) {
      // test for switch held down for > 2s
      if (swTO1 > 400) {
        Serial.println("SW1 long press!");
        doSw1Action(99);
        swCnt1 = 0;
      }
    } else {
      // time-out switch count action after 1s
      if (swTO1 > 200) {
        Serial.print("SW1 Cnt=");
        Serial.println(swCnt1);
        doSw1Action(swCnt1);
        swCnt1 = 0;
      }
    }
  }
  // read switch 2
  swRead2 = digitalRead(Pin_SW2);
  if (swRead2 != swState2) {
    // switch '2' has changed
    if (swRead2 == LOW) {swCnt2++; swTO2= 0;}
  } swState2 = swRead2; swTO2++;
  if (swRead2 == LOW) {ledState2 = HIGH; loopEn = 2000; setAllRGBLeds(0,0,4);}
  if (swCnt2 > 0) {
    if (swState2 == LOW) {
      // test for switch held down for > 2s
      if (swTO2 > 400) {
        Serial.println("SW2 long press!");
        doSw2Action(99);
        swCnt2 = 0;
      }
    } else {
      // time-out switch count action after 1s
      if (swTO2 > 200) {
        Serial.print("SW2 Cnt=");
        Serial.println(swCnt2);
        doSw2Action(swCnt2);
        swCnt2 = 0;
      }
    }
  }
  setLEDs (ledState0, ledState1, ledState2);
}

// ----------------------------------------------------------------------

void setAllRGBLeds (int zR, int zG, int zB) {
  for (int zLED = 0; zLED < 16; zLED++) {
    strip.setPixelColor(zLED,strip.Color(zR,zG,zB));
  } strip.show();
}

// ----------------------------------------------------------------------

void setColVal(int zP,long zC) {
  // set an array point zP to a specific Hex value zC
  colBlu[zP] = zC & 0xFF; colGrn[zP] = (zC>>8) & 0xFF; colRed[zP] = (zC>>16) & 0xFF;
}

// ----------------------------------------------------------------------

void SetDefaults() {
  // Initialise variables to default values
  autoMode = 0; // automode flag, 0=OFF, 1=back-off, 2=Autonomous
  autoTaskPnt = 0; // autoEngine main task pointer
  BTRx = 0; // > 1 if Bluetooth chardcater received
  BTxCnt = 0; // incremented value sent as 'LIVE' indicator
  BTxON = -1; // flag/counter to enable link 'LIVE' messaging
  cmdMode = ' '; // command mode G-gun, R-radar
  cmdType = ' '; // command mode type
  cmdVal = 0; // value associated with a cmdType
  doWait = 0; // pause in 10ms increments during do commands
  headInc = 1; // head movement increment value
  headSweep = -1; // > -1 sweeps head randomly
  headTgt = 45; // target head position
  intFlag = 0; // = 1 if interrupt has occured, default = 0
  JoyX = 128; // joypad move pointers
  JoyY = 128; // joypad move pointers
  ledState0 = LOW; // HIGH/LOW state of LED0 o/p
  ledState1 = LOW; // HIGH/LOW state of LED1 o/p
  ledState2 = LOW; // HIGH/LOW state of LED2 o/p
  legMode = 0; // mode affects height and stride options, default = 0
  loopEn = 0; // main loop enable, 0 = ON, >0 = OFF
  moved = 0; // counter > 0 indicating that a move has happened
  moveMode = 0; // 'Move' engine mode, 0 = RESET
  moveModeLast = -1; // previous 'Move' engine mode tracks transitions
  moveNext = 0; // the 'moveMode' to use after a pause
  movePause = 0; // 'Move' pause counter in loop cycles
  onceFlag = 1; // runs events once in POST
  raiseAngleVar = raiseAngle; // variable raise angle
  rangeFlag = 0; // range enable flag; > 0 ON, 0 = OFF default
  RGBEnh = 0; // >0 then show enhanced RGB patterns
  servoOE = 1; // servo board Output Enable; default = 1 OFF
  servoSwpCnt = 0; // target servo sweep counter
  servoSwpDelta = 102; // target servo sweep delta +/- 0 - 205
  servoSwpEn = 0; // target servo sweep enable; default = 0;
  servoSwpInc = 1; // target servo sweep increment +/- 1
  servoTgt = 0; // target servo for manual setting; default = 0
  speedTQ = 0; // movement speed mode, 0=tortoise, 1=quick
  swCnt0 = 0; // switch 0 counter
  swCnt1 = 0; // switch 1 counter
  swCnt2 = 0; // switch 2 counter
  swRead0 = HIGH; // read switch 0 state
  swRead1 = HIGH; // read switch 1 state
  swRead2 = HIGH; // read switch 2 state
  swState0 = HIGH; // most recent switch 0 state
  swState1 = HIGH; // most recent switch 1 state
  swState2 = HIGH; // most recent switch 2 state
  swTO0 = 0; // switch 0 time-out counter
  swTO0 = 1; // switch 1 time-out counter
  swTO0 = 2; // switch 2 time-out counter
  upDwnFlg = 0; // up/down state flag, 0 = down, 1 = up
  LoadDefArrays();
}

// ----------------------------------------------------------------------

void setLEDs (int z0, int z1, int z2) {
  // set the state of the three LEDs on the BT board  
  digitalWrite(Pin_LED0, z0); digitalWrite(Pin_LED1, z1); digitalWrite(Pin_LED2, z2);
}

// ----------------------------------------------------------------------

void SetPixiColours(long zC0,long zC1,long zC2,long zC3,long zC4,long zC5,long zC6,long zC7,long zC8,long zC9,long zCA,long zCB,long zCC,long zCD,long zCE,long zCF) {
  // sets ALL 16 RGB LEDs from provided colours
  anyA2 = zC0 & 0xFF; anyA1 = (zC0>>8) & 0xFF; anyA0 = (zC0>>16) & 0xFF;
  strip.setPixelColor(0,strip.Color(anyA0,anyA1,anyA2));
  anyA2 = zC1 & 0xFF; anyA1 = (zC1>>8) & 0xFF; anyA0 = (zC1>>16) & 0xFF;
  strip.setPixelColor(1,strip.Color(anyA0,anyA1,anyA2));
  anyA2 = zC2 & 0xFF; anyA1 = (zC2>>8) & 0xFF; anyA0 = (zC2>>16) & 0xFF;
  strip.setPixelColor(2,strip.Color(anyA0,anyA1,anyA2));
  anyA2 = zC3 & 0xFF; anyA1 = (zC3>>8) & 0xFF; anyA0 = (zC3>>16) & 0xFF;
  strip.setPixelColor(3,strip.Color(anyA0,anyA1,anyA2));
  anyA2 = zC4 & 0xFF; anyA1 = (zC4>>8) & 0xFF; anyA0 = (zC4>>16) & 0xFF;
  strip.setPixelColor(4,strip.Color(anyA0,anyA1,anyA2));
  anyA2 = zC5 & 0xFF; anyA1 = (zC5>>8) & 0xFF; anyA0 = (zC5>>16) & 0xFF;
  strip.setPixelColor(5,strip.Color(anyA0,anyA1,anyA2));
  anyA2 = zC6 & 0xFF; anyA1 = (zC6>>8) & 0xFF; anyA0 = (zC6>>16) & 0xFF;
  strip.setPixelColor(6,strip.Color(anyA0,anyA1,anyA2));
  anyA2 = zC7 & 0xFF; anyA1 = (zC7>>8) & 0xFF; anyA0 = (zC7>>16) & 0xFF;
  strip.setPixelColor(7,strip.Color(anyA0,anyA1,anyA2));
  anyA2 = zC8 & 0xFF; anyA1 = (zC8>>8) & 0xFF; anyA0 = (zC8>>16) & 0xFF;
  strip.setPixelColor(8,strip.Color(anyA0,anyA1,anyA2));
  anyA2 = zC9 & 0xFF; anyA1 = (zC9>>8) & 0xFF; anyA0 = (zC9>>16) & 0xFF;
  strip.setPixelColor(9,strip.Color(anyA0,anyA1,anyA2));
  anyA2 = zCA & 0xFF; anyA1 = (zCA>>8) & 0xFF; anyA0 = (zCA>>16) & 0xFF;
  strip.setPixelColor(10,strip.Color(anyA0,anyA1,anyA2));
  anyA2 = zCB & 0xFF; anyA1 = (zCB>>8) & 0xFF; anyA0 = (zCB>>16) & 0xFF;
  strip.setPixelColor(11,strip.Color(anyA0,anyA1,anyA2));
  anyA2 = zCC & 0xFF; anyA1 = (zCC>>8) & 0xFF; anyA0 = (zCC>>16) & 0xFF;
  strip.setPixelColor(12,strip.Color(anyA0,anyA1,anyA2));
  anyA2 = zCD & 0xFF; anyA1 = (zCD>>8) & 0xFF; anyA0 = (zCD>>16) & 0xFF;
  strip.setPixelColor(13,strip.Color(anyA0,anyA1,anyA2));
  anyA2 = zCE & 0xFF; anyA1 = (zCE>>8) & 0xFF; anyA0 = (zCE>>16) & 0xFF;
  strip.setPixelColor(14,strip.Color(anyA0,anyA1,anyA2));
  anyA2 = zCF & 0xFF; anyA1 = (zCF>>8) & 0xFF; anyA0 = (zCF>>16) & 0xFF;
  strip.setPixelColor(15,strip.Color(anyA0,anyA1,anyA2));
}

// ----------------------------------------------------------------------

void setRGBEnh() {
  // set the task for RGB enhanced mode depending on move direction
  if ((RGB > 0) && (RGB != 6)) {
    // only set mode if RGB engine has already been switched ON
    // and it is not set for head tracking mode 6
    if (RGBEnh > 0) {RGB = dirFlg + 1;} else {RGB = 1;}
    RGBmode = 0;
  }
}

// ----------------------------------------------------------------------

void SetServoAngle(int zServo, int zAngle) {
  // set the angle of a given servo
  // leg angles 0 - 130, shoulder angles 0 - 135
  // head angles 0 - 45 - 90
  // no test on valid range
  long zCal;
  servoAng[zServo] = zAngle; // record angle set
  switch (zServo) {
    case 0:
      zCal = servoA135[0]-servoA000[0]; zCal = ((zCal * zAngle)/90) + servoA000[0];
      I2Cpwm.setPWM(0, 0, zCal); break; // centre head
    case 1:
      zCal = servoA135[1]-servoA000[1]; zCal = ((zCal * zAngle)/135) + servoA000[1];
      I2Cpwm.setPWM(1, 0, zCal); break; // set left front shoulder
    case 2:
      zCal = servoA135[2]-servoA000[2]; zCal = ((zCal * zAngle)/113) + servoA000[2];
      I2Cpwm.setPWM(2, 0, zCal); break; // set left front leg
    case 3:
      zCal = servoA000[3]-servoA135[3]; zCal = servoA000[3] - ((zCal * zAngle)/135);
      I2Cpwm.setPWM(3, 0, zCal); break; // set right front shoulder
    case 4:
      zCal = servoA000[4]-servoA135[4]; zCal = servoA000[4] - ((zCal * zAngle)/113);
      I2Cpwm.setPWM(4, 0, zCal); break; // set right front leg
    case 5:
      zCal = servoA135[5]-servoA000[5]; zCal = ((zCal * zAngle)/135) + servoA000[5];
      I2Cpwm.setPWM(5, 0, zCal); break; // set left centre shoulder
    case 6:
      zCal = servoA135[6]-servoA000[6]; zCal = ((zCal * zAngle)/113) + servoA000[6];
      I2Cpwm.setPWM(6, 0, zCal); break; // set left centre leg
    case 7:
      zCal = servoA000[7]-servoA135[7]; zCal = servoA000[7] - ((zCal * zAngle)/135);
      I2Cpwm.setPWM(7, 0, zCal); break; // set right centre shoulder
    case 8:
      zCal = servoA000[8]-servoA135[8]; zCal = servoA000[8] - ((zCal * zAngle)/113);
      I2Cpwm.setPWM(8, 0, zCal); break; // set right centre leg
    case 9:
      zCal = servoA135[9]-servoA000[9]; zCal = ((zCal * zAngle)/135) + servoA000[9];
      I2Cpwm.setPWM(9, 0, zCal); break; // set left rear shoulder
    case 10:
      zCal = servoA135[10]-servoA000[10]; zCal = ((zCal * zAngle)/113) + servoA000[10];
      I2Cpwm.setPWM(10, 0, zCal); break; // set left rear leg
    case 11:
      zCal = servoA000[11]-servoA135[11]; zCal = servoA000[11] - ((zCal * zAngle)/135);
      I2Cpwm.setPWM(11, 0, zCal); break; // set right rear shoulder
    case 12:
      zCal = servoA000[12]-servoA135[12]; zCal = servoA000[12] - ((zCal * zAngle)/113);
      I2Cpwm.setPWM(12, 0, zCal); break; // set right rear leg
  }
}

// ----------------------------------------------------------------------

void SetServoFreq() {
  // manually set servo PWM frequence, noiminally 50 Hz  
  if (cmdVal < 45) cmdVal = 45; // block out of range values
  if (cmdVal > 65) cmdVal = 55; // block out of range values
  if (cmdVal != pwmFreq) {
    pwmFreq = cmdVal;
    I2Cpwm.setPWMFreq(pwmFreq);  // set PCA9685 PWM frequency
    Serial.print("PWM Freq = ");
    Serial.println(pwmFreq);
  }
}

// ----------------------------------------------------------------------

void SetServoOE(int zVal) {
  // manually set the servo board OE to HIGH or LOW
  if (zVal != 1) {
    servoOE = 0;
    JoyX = JoyCentre; JoyY = JoyCentre; // remove any residual demand
    digitalWrite(Pin_OE, LOW); // enable all PWM outputs
    Serial.println("ServoOE = LOW  [ON]");
  } else {
    servoOE = 1;
    digitalWrite(Pin_OE, HIGH); // disable all PWM outputs
    Serial.println("ServoOE = HIGH  [OFF]");
  }
}

// ----------------------------------------------------------------------

void SetServoStartPos(int zDel) {
  // set servos to default start positions
  // values determined from calibration tests
  // move head first
  SetServoAngle(0,45); delay(zDel);
  // now align shoulders, front to back
  SetServoAngle(1,0); SetServoAngle(3,0); delay(zDel);
  SetServoAngle(5,0); SetServoAngle(7,0); delay(zDel);
  SetServoAngle(9,0); SetServoAngle(11,0); delay(zDel);
  // now lower to the floor assuming starting from 'stand' height
  for (int zL=standAngle;zL>0;zL--) {
    anyDel = 0;
    if (servoAng[2]>zL) {SetServoAngle(2,zL); anyDel=1;}
    if (servoAng[4]>zL) {SetServoAngle(4,zL); anyDel=1;}
    if (servoAng[6]>zL) {SetServoAngle(6,zL); anyDel=1;}
    if (servoAng[8]>zL) {SetServoAngle(8,zL); anyDel=1;}
    if (servoAng[10]>zL) {SetServoAngle(10,zL); anyDel=1;}
    if (servoAng[12]>zL) {SetServoAngle(12,zL); anyDel=1;}
    if (anyDel > 0) {delay(15);}
  }
  // make sure it is on the ground
  SetServoAngle(2,0); SetServoAngle(4,0); SetServoAngle(6,0);
  SetServoAngle(8,0); SetServoAngle(10,0); SetServoAngle(12,0);
}

// ----------------------------------------------------------------------

void SetServoSweep() {
  // manually set target servo sweep mode  
  if (cmdVal != 1) {
    servoSwpEn = 0;
    Serial.println("ServoSweep = OFF");
  } else {
    servoSwpEn = 1;
    Serial.println("ServoSweep = ON");
  }
}

// ----------------------------------------------------------------------

void SetServoSwpDelta() {
  // manually set target servo sweep mode  
  if (cmdVal < 0) cmdVal = 0; // block out of range values
  if (cmdVal > 205) cmdVal = 205; // block out of range values
  if (cmdVal != servoSwpDelta) {
    servoSwpDelta = cmdVal;
    Serial.print("ServoSwpDelta = ");
    Serial.println(servoSwpDelta);
  }
}

// ----------------------------------------------------------------------

void SetServoTarget() {
  // set the target servo pointer 0 - 15 and report back
  if (cmdVal < 0) cmdVal = servoTgt; // block oiut of range values
  if (cmdVal > 15) cmdVal = servoTgt; // block oiut of range values
  if (cmdVal != servoTgt) {
    servoTgt = cmdVal;
    Serial.print("ServoTgt = ");
    Serial.println(servoTgt);
  }
}

// ----------------------------------------------------------------------

void SetServoValue() {
  // set the target servo value LL - UL and report back
  if ((cmdVal >= int(servoVLL[servoTgt])) && (cmdVal <= int(servoVUL[servoTgt]))) {
    // in range value received
    servoVal[servoTgt] = cmdVal;
    I2Cpwm.setPWM(servoTgt, 0, servoVal[servoTgt]);
    Serial.print("ServoVal = ");
    Serial.println(servoVal[servoTgt]);
  }
}

// ----------------------------------------------------------------------

void SetToLegPos(int zAngle) {
  // set ALL leg servos to a given sit angle 0 - 130
  // note leg is calibrated at 0 and 113 degrees
  // greater lift is allowed for walking over obstacles
  if ((zAngle >=0) && (zAngle <= 130)) {
    // value within limits
    SetServoAngle(2, zAngle); // set left front leg
    SetServoAngle(4, zAngle); // set right front leg
    SetServoAngle(6, zAngle); // set left centre leg
    SetServoAngle(8, zAngle); // set right centre leg
    SetServoAngle(10, zAngle); // set left rear leg
    SetServoAngle(12, zAngle); // set right rear leg
    Serial.print("Legs at = ");
    Serial.println(zAngle);
  }
}

// ----------------------------------------------------------------------

void SetToParkPos(int zAngle, int zC) {
  // set the shoulder servos to a given park angle 0 - 135
  if ((zAngle >=0) && (zAngle <= 135)) {
    // value within limits
    if (zC > 0) {SetServoAngle(0, 45);} // centre head
    SetServoAngle(1, zAngle); // set left front shoulder
    SetServoAngle(3, zAngle); // set right front shoulder
    SetServoAngle(5, zAngle); // set left centre shoulder
    SetServoAngle(7, zAngle); // set right centre shoulder
    SetServoAngle(9, zAngle); // set left rear shoulder
    SetServoAngle(11, zAngle); // set right rear shoulder
    Serial.print("Parked = ");
    Serial.println(zAngle);
  }
}

// ----------------------------------------------------------------------

void showColArray(int zOff) {
  // loads the colour array values into the strip and shows it, with an
  // offset zOff, clearing LEDs not included by the offset
  for (int zP = 0; zP < 16; zP++) {
    int zAdd = zP + zOff;
    if (zAdd >= 0 && zAdd <=15) {
      strip.setPixelColor(zP,strip.Color(colRed[zAdd], colGrn[zAdd], colBlu[zAdd]));
    } else {
      strip.setPixelColor(zP,strip.Color(0,0,0));
    }
  } strip.show();
}

// ----------------------------------------------------------------------

void showColMasked() {
  // loads the colour array values with no offset, but masks with
  // the range colours for LEDs 5 to 10  
  long zC0;
  for (int zP = 0; zP < 16; zP++) {
    if (zP >= 5 && zP <= 10) {
      // mask in range values
      zC0 = rangeLEDs[zP-5];
      anyA2 = zC0 & 0xFF; anyA1 = (zC0>>8) & 0xFF; anyA0 = (zC0>>16) & 0xFF;
      strip.setPixelColor(zP,strip.Color(anyA0,anyA1,anyA2));
    } else {
      // use normal colour array values
      strip.setPixelColor(zP,strip.Color(colRed[zP], colGrn[zP], colBlu[zP]));
    }
  } strip.show();
}

// ----------------------------------------------------------------------

void STOP() {
  // Emergency STOP command received
  digitalWrite(Pin_OE, HIGH); // disable all PWM outputs
  servoOE = 1; Serial.println("STOPPED!");
}

// ----------------------------------------------------------------------

void SweepTgtServo() {
  // called from main loop to sweep the target servo by delta
  uint16_t zVal;
  if (servoSwpInc > 0) {
    servoSwpCnt++;
    zVal = servoVal[servoTgt] + servoSwpCnt;
    if (zVal > servoVUL[servoTgt]) {zVal = servoVUL[servoTgt];}
  } else {
    servoSwpCnt--;
    zVal = servoVal[servoTgt] + servoSwpCnt;
    if (zVal < servoVLL[servoTgt]) {zVal = servoVLL[servoTgt];}
  }
  I2Cpwm.setPWM(servoTgt, 0, zVal);
  if (servoSwpCnt >= servoSwpDelta) {servoSwpInc = -1;}
  if (servoSwpCnt <= -servoSwpDelta) {servoSwpInc = 1;}
}

// ----------------------------------------------------------------------

void toggleBTx() {
  // toggle the auto-send link active flag
  if (BTxON >= 0) {
    BTxON = -1;
  } else {BTxON = 0;}
}

// ----------------------------------------------------------------------

void trigHCSR04 () {
  // sends a trigger pulse to the range sensor
  digitalWrite(Pin_Trig, LOW);
  delayMicroseconds(2); // ensure it is LOW to start with
  digitalWrite(Pin_Trig, HIGH);
  intMicros = micros(); // record time of pulse
  delayMicroseconds(10); // send 10us pulse
  digitalWrite(Pin_Trig, LOW);
  delayMicroseconds(100); // wait before enabling interrupt
  intFlag = 0; // prime interrupt flag
}

// ----------------------------------------------------------------------

void waitSwDwn() {
  // wait here whilst ANY switch is down
  while ((digitalRead(Pin_SW0)==LOW)||(digitalRead(Pin_SW1)==LOW)||(digitalRead(Pin_SW2)==LOW)) {
    // waiting
    delay(5);     
  }
}


