// ######################################################################
//
//  HexBot Sketch File v0.21 Beta
//
//  Released:  20/10/2016
//
//  Author: TechKnowTone
//
// ######################################################################
/*
    This software is furnished "as is", without technical support, and
    with no warranty, expressed or implied, as to its usefulness for any
    purpose.

    Elements of this software were taken from the public domain. It uses
    libraries which must be installed on your system.

    This includes changes to reduce LED power consumption, having had a
    regulator failure on the previous board.
    
    This version includes:
    * Autonomous modes
      * Back away
      * Chase
      * Track
      * Full autonomy
    * Bluetooth monitoring
    * Head movement
    * LEDs and Switch on Blue Tooth board
    * Leg stride and height settings
    * Variable rate /Move/ loop timing
    * Variable stride and height modes
    * RGB LED sequences
    * Smooth leg raise and lower    
    * Turning left and right on the spot
    * Ultrasonic ranging
    * Walking forwards
    * Walking forwards whilst turning left/right.
    * Walking backwards
    * Walking backwards whilst turning left/right.

    Commands:
    BNrc.   - button row/column
    PKnn.   - park ALL shoulders at a given angle, 0 - 135
    PLnn.   - set ALL legs to a given angle, 0 - 113
    TQnn.   - set 'Tortoise'=0 or 'Quick'=1 speed mode
    SDnn.   - set sweep delta +/- 0 - 300
    SFnn.   - set PWM frequency, nominally 50 Hz
    SOn.    - set servo board OE. 0 = LOW, 1 = HIGH
    SSn.    - set 'target' into sweep mode, 0 = OFF, 1 = ON
    STnn.   - set 'target' servo No. to nn
    SVnn.   - set 'target' servo PWM value to nn
    Xnn.    - set left/mid/right as 0-128-255
    Ynn.    - set forward/stop/reverse as 255-128-0
    Z       - EMERGENCY STOP!
    +       - Bluetooth 'live' received character
    !       - RESET

*/
// Declare library functions
#include <Wire.h>
#include <Adafruit_PWMServoDriver.h>
#include <SoftwareSerial.h> // Bluetooth library
#include <Adafruit_NeoPixel.h>

Adafruit_PWMServoDriver I2Cpwm = Adafruit_PWMServoDriver(0x40);

// define labels and constants
#define btRxPin 50 // Rx pin for Bluetooth
#define btTxPin 34 // Tx pin for Bluetooth
#define ColBlu 0x000088; // pallet colour
#define ColCya 0x004444; // pallet colour
#define ColGrn 0x008800; // pallet colour
#define ColPur 0x440044; // pallet colour
#define ColRed 0x880000; // pallet colour
#define ColYel 0x444400; // pallet colour
#define JoyCentreLL 98 // centre circle lower limit
#define JoyCentre 128 // centre circle value
#define JoyCentreUL 158 // centre circle lower limit
#define ledPin 15 // RGB LED serial pin
#define ledNum 16 // number of RGB LEDs connected
#define modeForward 20 // forward movement 'case'
#define modeBackward 21 // backward movement 'case'
#define modePause 99 // pause movement 'case'
#define modeToRest 9 // pause movement 'case'
#define modeTurnRht 22 // turn right movement 'case'
#define modeTurnLft 23 // turn leftt movement 'case'
#define objSteer 35 // lower range limit in cm for triggering object TURN
#define objTrig 25 // lower range limit in cm for triggering object STOP!
#define Pin_Echo 2 // define ultrasonic echo pin
#define Pin_LED0 9 // LED0 pin - Red LED
#define Pin_LED1 8 // LED1 pin - Green LED
#define Pin_LED2 10 // LED2 pin - Blue LED
#define Pin_OE 19 // define PCA9685 OE pin
#define Pin_SW0 11 // switch SW0 input
#define Pin_SW1 12 // switch SW1 input
#define Pin_SW2 13 // switch SW2 input
#define Pin_Trig 5 // define ultrasonic trigger pin
#define raiseAngle 112 // default leg angle for 'Raised' leg
#define raiseAngleLL 75 // LL leg angle for 'Raised' leg
#define restAngle 65 // default shoulder angle for 'Rest' position
#define restHeight 38 // default leg angle for 'Rest' position
#define standAngle 38 // default shoulder angle for 'Stand' position
#define strideAngle 50 // leg angle when walking

// NeoPixel parameters
// Parameter 1 = number of pixels in strip
// Parameter 2 = pin number (most are valid)
// Parameter 3 = pixel type flags, add together as needed:
//   NEO_RGB     Pixels are wired for RGB bitstream
//   NEO_GRB     Pixels are wired for GRB bitstream
//   NEO_KHZ400  400 KHz bitstream (e.g. FLORA pixels)
//   NEO_KHZ800  800 KHz bitstream (e.g. High Density LED strip)
//Adafruit_NeoPixel strip = Adafruit_NeoPixel(ledNum, ledPin, NEO_GRB + NEO_KHZ800);
Adafruit_NeoPixel strip = Adafruit_NeoPixel(ledNum, ledPin, NEO_GRB + NEO_KHZ800);
SoftwareSerial BTserial(btRxPin, btTxPin, false); // Rx, Tx to BT_Tx, BT_Rx

// Declare and initialise global variables
long anyA0; // used as temp angle
long anyA1; // used as temp angle
long anyA2; // used as temp angle
int anyCol; // any colour
int anyDel; // any temp delay value
float anyFP; // as temp FP angle
long anyVal; // used as temp variable
int anyCalc; // used as temp variable
int autoBodyPCnt; // autonomous body pause counter
int autoBodyPnt; // autonomous body movement pointer
int autoHeadAng; // autonomous head angle pointer
int autoHeadAngLast; // autonomous head angle in previous cycle
int autoHeadCnt; // autonomous head movement counter
int autoHeadDel; // autonomouse head movement inc delay in 5ms
int autoHeadInc; // autonomous head angle increment
int autoHeadPhase; // sweep phase counter for R/L/R/L motion
int autoHeadPnt; // autonomous head task pointer
int autoMode; // automode flag, 0=OFF, 1=back-off, 2=Autonomous
int autoSteerCnt; // counter reducing steering offset
int autoTaskPnt; // autoEngine main task pointer
int bluetoothCnt; // Bluetooth LED colour count down
int btVal; // value received on Bluetooth port
int BTRx; // > 1 if Bluetooth character received
int BTxCnt; // incremented value sent as 'LIVE' indicator
int BTxON; // flag/counter to enable link 'LIVE' messaging
char cmdMode; // command mode G-gun, R-radar
char cmdType; // command mode type
int cmdVal; // value associated with a cmdType
uint8_t colBlu[] = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0}; //colour blue
uint8_t colGrn[] = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0}; //colour green
uint8_t colRed[] = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0}; //colour red
int doWait; // pause in 10ms increments during do commands
int dirFlg; // move engine direction flag, 0=off, 1=forward, 2=Rht turn, 3=backward, 4=Lft turn
int echoTO; // echo time-out counter for range measurements
int headInc; // head movement increment value
int headMax, headMin; // min/max range angles
int headSweep; // > -1 sweeps head randomly
int headTgt; // target head position
volatile int intFlag; // = 1 if interrupt has occured, default = 0
volatile unsigned long intMicros; // time when interrrup occured
int JoyX, JoyY; // joypad move pointers
char keyChar; // any keyboard character
int keyVal; // any keyboard value
int ledState0; // HIGH/LOW state of LED0 o/p
int ledState1; // HIGH/LOW state of LED1 o/p
int ledState2; // HIGH/LOW state of LED2 o/p
int legMode; // mode affects height and stride options
int loopEn; // main loop enable, 0 = ON, >0 = OFF
int lowerIncCnt; // a counter for smooth lowering of legs
long moveAngle; // primary angle moved in walking
float moveAngleFP; // secondary angle moved in walking
float moveAngleMu; // multiplier angle moved in walking
float moveAnglePr; // primary angle moved in walking
int moved; // counter > 0 indicating that a move has happened
float moveIncFP; // incremental angle moved in walking
unsigned long moveInterval = 5000; // main loop interval in microseconds
int moveMode; // 'Move' engine mode
int moveModeLast; // previous 'Move' engine mode tracks transitions
int moveNext; // the 'moveMode' to use after a pause
int movePause; // 'Move' pause counter in loop cycles
int moveRest; // move to 'Rest' task pointer
int moveSide; // defines which side left/right is primary mover
int moveWalk; // move 'Walking' task pointer
unsigned long next5ms; // main loop 5ms interval timer
unsigned long nextMove; // move loop interval target in microseconds
int onceFlag = 1; // if > 0 runs events once in POST
float pwmFreq = 52; // MG90S servos run at ~50 Hz updates
long raiseAngleVar = raiseAngle; // variable raise angle
int raiseIncCnt; // a counter for smooth raising of leg
unsigned long rangeArray[] = {0,0,0,0,0,0}; // head ranging data
int rangeFlag; // range enable flag; > 0 ON, 0 = OFF default
int rangeLedON; // >0 if a range value is detected
long rangeLEDs[] = {0,0,0,0,0,0}; // head LED lamps
int rangeTIR; // target in range flag, > 0 yes, < 1 no
long rangeVal; // determined range distance in cm
int RGB; // task enable/pointer flag, 0 = OFF default, >0 = ON 
int RGBCntr; // any temp counter
int RGBEnh; // >0 then show enhanced RGB patterns
int RGBmode; // RGB subtask pointer
int RGBSteerFlag; // used to miss cycles when steering
int RGBStrDir; // used to track steering and re-synch
int servoSwpCnt; // target servo sweep counter
int servoSwpDelta; // target servo sweep delta +/- 0 - 205
int servoSwpEn; // target servo sweep enable; default = 0
int servoSwpInc; // target servo sweep increment +/- 1
int servoTgt; // target/pointer servo for manual setting
int servoOE; // servo board Output Enable; default = 1  OFF
int speedTQ; // movement speed mode, 0=tortoise, 1=quick
int steerFlag; // direction flag used in steering, -1=left, 0=none, 1=right
long strideAngleVar = strideAngle; // variable stride angle
int swCnt0, swCnt1, swCnt2; // switch counters
int swRead0, swRead1, swRead2; // read switch state
int swState0, swState1, swState2; // most recent switch state
int swTO0, swTO1, swTO2; // switch time-out counters
uint8_t twbrbackup = TWBR;  // save I2C bitrate
int upDwnFlg; // up/down state flag, 0 = down, 1 = up

// define arrays using 'calibrated' PWM values for angle calcs
//             Servo Ch:  000,001,002,003,004,005,006,007,008,009,010,011,012,013,014,015
uint16_t servoA000[16] = {258,214,187,476,486,194,176,504,486,195,189,467,488,247,247,247}; // calibrated servo values for 0 degrees setting
uint16_t servoA135[16] = {442,492,427,197,253,470,416,212,245,470,420,185,251,247,247,247}; // calibrated servo values for max (135/113) degrees setting
int servoAng[16] = {45,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0}; // angle used to set a servo
uint16_t servoVal[16]; // target servo value for manual setting
uint16_t servoVLL[16]; // target servo value LL for manual setting
uint16_t servoVUL[16]; // target servo value UL for manual setting

void setup() {
  // put your setup code here, to run once:
  pinMode(Pin_OE, OUTPUT);
  digitalWrite(Pin_OE, HIGH); // disable all PWM outputs to start with
  I2Cpwm.begin();
  I2Cpwm.setPWMFreq(pwmFreq);  // set PCA9685 PWM frequency
  TWBR = 12; // upgrade to I2C bitrate to 400KHz!
  SetDefaults(); // initialise variables
  Serial.begin(19200);
  pinMode(Pin_SW0, INPUT_PULLUP);
  pinMode(Pin_SW1, INPUT_PULLUP);
  pinMode(Pin_SW2, INPUT_PULLUP);
  pinMode(Pin_LED0, OUTPUT);
  digitalWrite(Pin_LED0, LOW);
  pinMode(Pin_LED1, OUTPUT);
  digitalWrite(Pin_LED1, LOW);
  pinMode(Pin_LED2, OUTPUT);
  digitalWrite(Pin_LED2, LOW);
  BTserial.begin(9600); // start BT serial comms
  BTserial.listen();
  strip.begin();
  strip.show(); // Initialize all pixels to 'off'
  pinMode(Pin_Trig, OUTPUT); // ultrasonic Trig pin
  digitalWrite(Pin_Trig, LOW);
  attachInterrupt(digitalPinToInterrupt(Pin_Echo), echoInt, FALLING);
  RunPOST();
}

void loop() {
  // put your main code here, to run repeatedly:
  if (micros() > next5ms) {
    // events occuring every 5ms
    next5ms = micros() + 5000;
    if (loopEn < 1) {
      if (autoMode == 4) {autoEngine();}
      if (RGB > 0) {RGBtasks();}
      if (servoSwpEn > 0) {SweepTgtServo();} // in 16-channel mode sweeping
      if (moved > 0) {moved--; if (moved < 1) {moveRest = 0; moveMode = 9;}}
      if (headSweep >= 0) {moveHead();}
    } else {loopEn--;} // count down inhibit timer until < 1
    if (BTRx > 0) {bluetoothCnt--; if (bluetoothCnt < 1) {Reset();}}
    scanSwitches();
  }
  if (micros() > nextMove) {
    // in 'Move' modes with variable loop rates
    nextMove = micros() + moveInterval;
    if (loopEn < 1) {doMoveModes();}
  }  
  readKey(); readBTChar();
}

// ----------------------------------------------------------------------

/* Strings for serial monitor

Set head and shoulder servos to 450:
  st0.sv450.st1.sv450.st3.sv450.st5.sv450.st7.sv450.st9.sv450.st11.sv450.


*/



